package de.fraunhofer.iem.dataprovider.sarif

import de.fraunhofer.iem.dataprovider.toolResult.*
import de.fraunhofer.iem.dataprovider.toolResult.Rule
import de.fraunhofer.iem.dataprovider.toolResult.Tool
import kotlinx.serialization.decodeFromString
import kotlinx.serialization.json.Json
import java.io.IOException
import java.nio.file.Path
import java.sql.Timestamp
import java.time.Instant
import java.util.*

fun de.fraunhofer.iem.dataprovider.sarif.Rule.asDbObject(): de.fraunhofer.iem.dataprovider.toolResult.Rule {
    val rule = Rule()
    rule.ruleId = this.id
    rule.shortDescription = this.shortDescription.text
    return rule
}


fun de.fraunhofer.iem.dataprovider.sarif.Tool.asDbObject(): Tool {
    val tool = Tool()
    val rules = this.driver.rules.map {
        it.asDbObject()
    }
    tool.rules = rules.toMutableSet()
    tool.name = this.driver.fullName
    tool.version = this.driver.version
    return tool
}

fun Location.asDbObject(): ResultLocation {
    val resultLocation = ResultLocation()
    resultLocation.endColumn = this.physicalLocation.region.endColumn
    resultLocation.endLine = this.physicalLocation.region.endLine
    resultLocation.startColumn = this.physicalLocation.region.startColumn
    resultLocation.startLine = this.physicalLocation.region.startLine

    resultLocation.artifactLocation = this.physicalLocation.artifactLocation.uri
    return resultLocation
}

fun Result.asDbObject(): ToolResult {
    val toolResult = ToolResult()
    when (this.level.uppercase(Locale.getDefault())) {
        "WARNING" -> toolResult.level = Level.Warning
        "ERROR" -> toolResult.level = Level.Error
        "NOTE" -> toolResult.level = Level.Note
        else -> toolResult.level = Level.None
    }

    toolResult.message = this.message.text
    val locations = this.locations.map { it.asDbObject() }
    toolResult.resultLocations = locations.toMutableSet()

    return toolResult
}

fun Sarif.asDbObject(): List<ToolRun> {


    return this.runs.map { run ->
        val tr = ToolRun()
        val tool = run.tool.asDbObject()
        tr.tool = tool
        tr.timeStamp = Timestamp.from(Instant.now())
        val results = run.results.map { it.asDbObject() }
        tr.toolResults = results.toMutableSet()
        tr

    }
}

fun getSarifFromFilePath(resultPath: Path): Sarif {
    val resFile = resultPath.toFile()
    if (resFile.exists()) {
        val resString = resFile.readText()
        val json = Json { ignoreUnknownKeys = true }
        return json.decodeFromString<Sarif>(resString)
    }

    throw IOException("File not found / Path is no file.")
}
