package de.fraunhofer.iem.dataprovider.taskManager.tasks

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.nio.file.Files
import java.nio.file.Paths

sealed class ProcessTask : Task() {
    protected abstract val flags: Array<String>
    protected abstract val execPath: String


    private val mainScope = CoroutineScope(Dispatchers.Default)
    override suspend fun execute() {
        if (isExecutable(execPath)) {
            val process = ProcessBuilder(execPath, *flags).start()

            process.onExit().thenApply { p1 ->
                mainScope.launch {
                    handleProcessReturn(p1)
                }
            }
        } else {
            logger.warn("Given execPath is not an executable $execPath.")
            // TODO: we should probably throw an exception here.
        }
    }

    private fun isExecutable(filePath: String): Boolean {
        val path = Paths.get(filePath)
        return Files.isExecutable(path)
    }

    abstract suspend fun handleProcessReturn(p: Process)
}
