package de.fraunhofer.iem.dataprovider.toolResult

import de.fraunhofer.iem.dataprovider.gitlab.Repository
import jakarta.persistence.*
import org.hibernate.Hibernate
import org.hibernate.annotations.JdbcTypeCode
import org.hibernate.type.SqlTypes
import java.sql.Timestamp
import java.util.*





@Entity
@Table(name = "tool_run")
class ToolRun {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @OneToMany(mappedBy = "toolRun", orphanRemoval = true)
    var toolResults: MutableSet<ToolResult> = mutableSetOf()

    @OneToOne(orphanRemoval = true)
    @JoinColumn(name = "tool_id")
    var tool: Tool? = null

    @JdbcTypeCode(SqlTypes.TIMESTAMP)
    @Column(name = "time_stamp")
    var timeStamp: Timestamp? = null

    @ManyToOne
    @JoinColumn(name = "repository_id")
    var repository: Repository? = null
}

@Entity
@Table(name = "tool")
class Tool {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @OneToMany(mappedBy = "tool", orphanRemoval = true)
    var rules: MutableSet<Rule> = mutableSetOf()

    @Column(name = "name")
    var name: String? = null

    @Column(name = "version")
    var version: String? = null
}

enum class Level {
    Warning, Error, Note, None
}

enum class Kind {
    pass, open, informational, notApplicable
}

@Entity
@Table(name = "tool_result")
class ToolResult {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "tool_run_id")
    var toolRun: ToolRun? = null

    @OneToMany(mappedBy = "toolResult", orphanRemoval = true)
    var resultLocations: MutableSet<ResultLocation> = mutableSetOf()

    @Enumerated
    @Column(name = "level")
    var level: Level? = null

    @Enumerated
    @Column(name = "kind")
    var kind: Kind? = null

    @Column(name = "message")
    var message: String? = null

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as ToolResult

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}

@Entity
@Table(name = "result_location")
class ResultLocation {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "tool_result_id")
    var toolResult: ToolResult? = null

    @OneToOne(orphanRemoval = true)
    @JoinColumn(name = "rule_id")
    var rule: Rule? = null

    @Column(name = "artifact_location")
    var artifactLocation: String? = null

    @Column(name = "end_column")
    var endColumn: Int? = null

    @Column(name = "end_line")
    var endLine: Int? = null

    @Column(name = "start_column")
    var startColumn: Int? = null

    @Column(name = "start_line")
    var startLine: Int? = null

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as ResultLocation

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}

@Entity
@Table(name = "rule")
class Rule {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id", nullable = false)
    var id: UUID? = null

    @ManyToOne
    @JoinColumn(name = "tool_id")
    var tool: Tool? = null


    @Column(name = "rule_id")
    var ruleId: String? = null


    @Column(name = "short_description")
    var shortDescription: String? = null


    @ElementCollection
    @CollectionTable(name = "rule_messages", joinColumns = [JoinColumn(name = "owner_id")])
    @Column(name = "message")
    var messages: MutableList<String> = mutableListOf()

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (other == null || Hibernate.getClass(this) != Hibernate.getClass(other)) return false
        other as Rule

        return id != null && id == other.id
    }

    override fun hashCode(): Int = javaClass.hashCode()
}